""" Code to read Raymetrics version of Licel binary files."""
import datetime
import logging

import numpy as np
import pytz

from .licel import LicelFile, LicelLidarMeasurement, LicelChannel, PhotodiodeChannel

logger = logging.getLogger(__name__)


class ScanningFile(LicelFile):
    licel_file_header_format = ['filename',
                                'start_date start_time end_date end_time altitude longitude latitude zenith_angle azimuth_angle temperature pressure',
                                # Appart from Site that is read manually
                                'azimuth_start azimuth_stop azimuth_step zenith_start zenith_finish zenith_step azimuth_offset',
                                'LS1 rate_1 LS2 rate_2 number_of_datasets', ]
    licel_file_channel_format = 'active analog_photon laser_used number_of_datapoints 1 HV bin_width wavelength d1 d2 d3 d4 ADCbits number_of_shots discriminator ID'

    def _read_rest_of_header(self, f):
        """ Read the rest of the header lines, after line 2. """
        raw_info = {}

        third_line = f.readline()
        raw_info.update(self.match_lines(third_line, self.licel_file_header_format[2]))

        fourth_line = f.readline()
        raw_info.update(self.match_lines(fourth_line, self.licel_file_header_format[3]))
        return raw_info

    def _assign_properties(self):
        super(ScanningFile, self)._assign_properties()
        self.azimuth_angle = float(self.raw_info['altitude'])
        self.temperature = float(self.raw_info['temperature'])
        self.pressure = float(self.raw_info['pressure'])
        self.azimuth_start = float(self.raw_info['azimuth_start'])
        self.azimuth_stop = float(self.raw_info['azimuth_stop'])
        self.azimuth_step = float(self.raw_info['azimuth_step'])
        self.zenith_start = float(self.raw_info['zenith_start'])
        self.zenith_finish = float(self.raw_info['zenith_finish'])
        self.zenith_step = float(self.raw_info['zenith_step'])
        self.azimuth_offset = float(self.raw_info['azimuth_offset'])


class ScanningChannel(LicelChannel):
    def __init__(self):
        super(ScanningChannel, self).__init__()

        self.azimuth_start = None
        self.azimuth_stop = None
        self.azimuth_step = None
        self.zenith_start = None
        self.zenith_finish = None
        self.zenith_step = None
        self.azimuth_offset = None
        self.zenith_angles = []
        self.azimuth_angles = []

    def append_file(self, current_file, file_channel):
        super(ScanningChannel, self).append_file(current_file, file_channel)
        self.zenith_angles.append(current_file.zenith_angle)
        self.azimuth_angles.append(current_file.azimuth_angle)

    def _assign_properties(self, current_file, file_channel):
        super(ScanningChannel, self)._assign_properties(current_file, file_channel)
        self._assign_unique_property('azimuth_start', current_file.azimuth_start)
        self._assign_unique_property('azimuth_stop', current_file.azimuth_stop)
        self._assign_unique_property('azimuth_step', current_file.azimuth_step)
        self._assign_unique_property('zenith_start', current_file.zenith_start)
        self._assign_unique_property('zenith_finish', current_file.zenith_finish)
        self._assign_unique_property('zenith_step', current_file.zenith_step)


class ScanningLidarMeasurement(LicelLidarMeasurement):
    file_class = ScanningFile
    channel_class = ScanningChannel
    photodiode_class = PhotodiodeChannel
